@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2006 - 2018
+  Copyright (c) 2006 - 2020
   CLST - Radboud University
   ILK  - Tilburg University
 
@@ -43,6 +43,46 @@
 
 using namespace std;
 using namespace Tokenizer;
+using TiCC::operator<<;
+
+string fix_639_1( const string& language ){
+  string result = language;
+  // support some backward compatability to old ISO 639-1 codes
+  if ( language == "nl" ){
+    result = "nld";
+  }
+  else if ( language == "de" ){
+    result = "deu";
+  }
+  else if ( language == "fr" ){
+    result = "fra";
+  }
+  else if ( language == "pt" ){
+    result = "por";
+  }
+  else if ( language == "es" ){
+    result = "spa";
+  }
+  else if ( language == "fy" ){
+    result = "fry";
+  }
+  else if ( language == "se" ){
+    result = "swe";
+  }
+  else if ( language == "en" ){
+    result = "eng";
+  }
+  else if ( language == "it" ){
+    result = "ita";
+  }
+  else if ( language == "ru" ){
+    result = "rus";
+  }
+  else if ( language == "tr" ){
+    result = "tur";
+  }
+  return result;
+}
 
 void usage(){
   set<string> languages = Setting::installed_languages();
@@ -76,6 +116,7 @@ void usage(){
        << "\t                    'full' - add text to all levels: <p> <s> <w> etc." << endl
        << "\t                    'minimal' - don't introduce text on higher levels, but retain what is already there." << endl
        << "\t                    'none' - only introduce text on <w>, AND remove all text from higher levels" << endl
+       << "\t--allow-word-corrections   - allow tokenization of FoLiA Word elements." << endl
        << "\t--filterpunct     - remove all punctuation from the output" << endl
        << "\t--uselanguages=<lang1,lang2,..langn> - Using FoLiA input, only tokenize strings in these languages. Default = 'lang1'" << endl
        << "\t--detectlanguages=<lang1,lang2,..langn> - try to assign a language to each line of text input. Default = 'lang1'" << endl
@@ -85,7 +126,7 @@ void usage(){
        << "\t-Q                - Enable quote detection (experimental)" << endl
        << "\t-V or --version   - Show version information" << endl
        << "\t-x <DocID>        - Output FoLiA XML, use the specified Document ID (obsolete)" << endl
-       << "\t-F                - Input file is in FoLiA XML. All untokenised sentences will be tokenised." << endl
+       << "\t-F                - Input file is in FoLiA XML. All untokenized sentences will be tokenized." << endl
        << "\t                    -F is automatically set when inputfile has extension '.xml'" << endl
        << "\t-X                - Output FoLiA XML, use the Document ID specified with --id=" << endl
        << "\t--id <DocID>      - use the specified Document ID to label the FoLia doc." << endl
@@ -110,6 +151,7 @@ int main( int argc, char *argv[] ){
   bool xmlin = false;
   bool xmlout = false;
   bool verbose = false;
+  bool docorrectwords = false;
   string redundancy = "minimal";
   string eosmarker = "<utt>";
   string docid = "untitleddoc";
@@ -122,14 +164,17 @@ int main( int argc, char *argv[] ){
   string ifile;
   string ofile;
   string c_file;
-  bool passThru = false;
+  bool pass_thru = false;
   bool sentencesplit = false;
   string norm_set_string;
   string add_tokens;
-
+  string command_line = "ucto";
+  for ( int i=1; i < argc; ++i ){
+    command_line += " " + string(argv[i]);
+  }
   try {
     TiCC::CL_Options Opts( "d:e:fhlPQunmN:vVL:c:s:x:FXT:",
-			   "filter:,filterpunct,passthru,textclass:,inputclass:,outputclass:,normalize:,id:,version,help,detectlanguages:,uselanguages:,textredundancy:,add-tokens:,split");
+			   "filter:,filterpunct,passthru,textclass:,inputclass:,outputclass:,normalize:,id:,version,help,detectlanguages:,uselanguages:,textredundancy:,add-tokens:,split,allow-word-corrections");
     Opts.init(argc, argv );
     if ( Opts.extract( 'h' )
 	 || Opts.extract( "help" ) ){
@@ -139,7 +184,7 @@ int main( int argc, char *argv[] ){
     if ( Opts.extract( 'V' ) ||
 	 Opts.extract( "version" ) ){
       cout << "Ucto - Unicode Tokenizer - version " << Version() << endl
-	   << "(c) CLST 2015 - 2018, Centre for Language and Speech Technology, Radboud University Nijmegen" << endl
+	   << "(c) CLST 2015 - 2020, Centre for Language and Speech Technology, Radboud University Nijmegen" << endl
 	   << "(c) ILK 2009 - 2015, Induction of Linguistic Knowledge Research Group, Tilburg University" << endl
 	   << "Licensed under the GNU General Public License v3" << endl;
       cout << "based on [" << folia::VersionName() << "]" << endl;
@@ -147,6 +192,7 @@ int main( int argc, char *argv[] ){
     }
     Opts.extract('e', inputEncoding );
     dopunctfilter = Opts.extract( "filterpunct" );
+    docorrectwords = Opts.extract( "allow-word-corrections" );
     paragraphdetection = !Opts.extract( 'P' );
     xmlin = Opts.extract( 'F' );
     quotedetection = Opts.extract( 'Q' );
@@ -184,7 +230,6 @@ int main( int argc, char *argv[] ){
       }
       //      sentenceperlineoutput = true;
     }
-    passThru = Opts.extract( "passthru" );
     string textclass;
     Opts.extract( "textclass", textclass );
     Opts.extract( "inputclass", inputclass );
@@ -212,7 +257,10 @@ int main( int argc, char *argv[] ){
       }
       dofiltering = result;
     }
-    if ( dofiltering && xmlin && outputclass == inputclass ){
+    if ( dofiltering
+	 && xmlin
+	 && outputclass == inputclass
+	 && !docorrectwords ){
       // we cannot mangle the original inputclass, so disable filtering
       cerr << "ucto: --filter=NO is automatically set. inputclass equals outputclass!"
 	   << endl;
@@ -234,9 +282,22 @@ int main( int argc, char *argv[] ){
 	throw TiCC::OptionError( "invalid value for -d: " + value );
       }
     }
+    pass_thru = Opts.extract( "passthru" );
     bool use_lang = Opts.is_present( "uselanguages" );
     bool detect_lang = Opts.is_present( "detectlanguages" );
+    if ( detect_lang && use_lang ){
+      throw TiCC::OptionError( "--detectlanguages and --uselanguages options conflict. Use only one of these." );
+    }
+    if ( use_lang && pass_thru ){
+      throw TiCC::OptionError( "--passtru an --uselanguages options conflict. Use only one of these." );
+    }
+    if ( detect_lang && pass_thru ){
+      throw TiCC::OptionError( "--passtru an --detectlanguages options conflict. Use only one of these." );
+    }
     if ( Opts.is_present('L') ) {
+      if ( pass_thru ){
+	throw TiCC::OptionError( "--passtru an -L options conflict. Use only one of these." );
+      }
       if ( Opts.is_present('c') ){
 	throw TiCC::OptionError( "-L and -c options conflict. Use only one of these." );
       }
@@ -255,65 +316,31 @@ int main( int argc, char *argv[] ){
 	throw TiCC::OptionError( "-c and --uselanguages options conflict. Use only one of these." );
       }
     }
-    if ( detect_lang && use_lang ){
-      throw TiCC::OptionError( "--detectlanguages and --uselanguages options conflict. Use only one of these." );
-    }
     Opts.extract( 'c', c_file );
 
-    string languages;
-    Opts.extract( "detectlanguages", languages );
-    if ( languages.empty() ){
-      Opts.extract( "uselanguages", languages );
-    }
-    else {
-      do_language_detect = true;
-    }
-    if ( !languages.empty() ){
-      if ( TiCC::split_at( languages, language_list, "," ) < 1 ){
-	throw TiCC::OptionError( "invalid language list: " + languages );
+    if ( !pass_thru ){
+      string languages;
+      Opts.extract( "detectlanguages", languages );
+      if ( languages.empty() ){
+	Opts.extract( "uselanguages", languages );
       }
-    }
-    else {
-      // so --detectlanguages or --uselanguages
-      string language;
-      if ( Opts.extract('L', language ) ){
-	// support some backward compatability to old ISO 639-1 codes
-	if ( language == "nl" ){
-	  language = "nld";
-	}
-	else if ( language == "de" ){
-	  language = "deu";
-	}
-	else if ( language == "fr" ){
-	  language = "fra";
-	}
-	else if ( language == "pt" ){
-	  language = "por";
-	}
-	else if ( language == "es" ){
-	  language = "spa";
-	}
-	else if ( language == "fy" ){
-	  language = "fry";
-	}
-	else if ( language == "se" ){
-	  language = "swe";
-	}
-	else if ( language == "en" ){
-	  language = "eng";
-	}
-	else if ( language == "it" ){
-	  language = "ita";
-	}
-	else if ( language == "ru" ){
-	  language = "rus";
-	}
-	else if ( language == "tr" ){
-	  language = "tur";
+      else {
+	do_language_detect = true;
+      }
+      if ( !languages.empty() ){
+	if ( TiCC::split_at( languages, language_list, "," ) < 1 ){
+	  throw TiCC::OptionError( "invalid language list: " + languages );
 	}
       }
-      if ( !language.empty() ){
-	language_list.push_back( language );
+      else {
+	// so NOT --detectlanguages or --uselanguages
+	string language;
+	if ( Opts.extract('L', language ) ){
+	  language = fix_639_1( language );
+	}
+	if ( !language.empty() ){
+	  language_list.push_back( language );
+	}
       }
     }
     Opts.extract("normalize", norm_set_string );
@@ -331,6 +358,9 @@ int main( int argc, char *argv[] ){
     if ( use_lang && !xmlin ){
       throw TiCC::OptionError( "--uselanguages is only valid for FoLiA input" );
     }
+    if ( docorrectwords && !xmlin ){
+      throw TiCC::OptionError( "--allow-word-corrections is only valid for FoLiA input" );
+    }
     if ( files.size() == 2 ){
       ofile = files[1];
       if ( TiCC::match_back( ofile, ".xml" ) ){
@@ -340,6 +370,7 @@ int main( int argc, char *argv[] ){
     if ( files.size() > 2 ){
       cerr << "found additional arguments on the commandline: " << files[2]
 	   << "...." << endl;
+      return EXIT_FAILURE;
     }
   }
   catch( const TiCC::OptionError& e ){
@@ -347,7 +378,7 @@ int main( int argc, char *argv[] ){
     usage();
     return EXIT_FAILURE;
   }
-  if ( !passThru ){
+  if ( !pass_thru ){
     set<string> available_languages = Setting::installed_languages();
     if ( !c_file.empty() ){
       cfile = c_file;
@@ -434,12 +465,34 @@ int main( int argc, char *argv[] ){
       return EXIT_FAILURE;
     }
   }
-
   try {
     TokenizerClass tokenizer;
     // set debug first, so init() can be debugged too
     tokenizer.setDebug( debug );
-    if ( passThru ){
+    tokenizer.set_command( command_line );
+    tokenizer.setEosMarker( eosmarker );
+    tokenizer.setVerbose( verbose );
+    tokenizer.setSentenceSplit(sentencesplit);
+    tokenizer.setSentencePerLineOutput(sentenceperlineoutput);
+    tokenizer.setSentencePerLineInput(sentenceperlineinput);
+    tokenizer.setLowercase(tolowercase);
+    tokenizer.setUppercase(touppercase);
+    tokenizer.setNormSet(norm_set_string);
+    tokenizer.setParagraphDetection(paragraphdetection);
+    tokenizer.setQuoteDetection(quotedetection);
+    tokenizer.setNormalization( normalization );
+    tokenizer.setInputEncoding( inputEncoding );
+    tokenizer.setFiltering(dofiltering);
+    tokenizer.setWordCorrection(docorrectwords);
+    tokenizer.setLangDetection(do_language_detect);
+    tokenizer.setPunctFilter(dopunctfilter);
+    tokenizer.setInputClass(inputclass);
+    tokenizer.setOutputClass(outputclass);
+    tokenizer.setXMLOutput(xmlout, docid);
+    tokenizer.setXMLInput(xmlin);
+    tokenizer.setTextRedundancy(redundancy);
+
+    if ( pass_thru ){
       tokenizer.setPassThru( true );
     }
     else {
@@ -463,35 +516,24 @@ int main( int argc, char *argv[] ){
 	}
 	return EXIT_FAILURE;
       }
+      if ( !cfile.empty() ){
+	cerr << "ucto: configured from file: " << cfile << endl;
+      }
+      else {
+	cerr << "ucto: configured for languages: " << language_list << endl;
+      }
     }
 
-    tokenizer.setEosMarker( eosmarker );
-    tokenizer.setVerbose( verbose );
-    tokenizer.setSentenceSplit(sentencesplit);
-    tokenizer.setSentencePerLineOutput(sentenceperlineoutput);
-    tokenizer.setSentencePerLineInput(sentenceperlineinput);
-    tokenizer.setLowercase(tolowercase);
-    tokenizer.setUppercase(touppercase);
-    tokenizer.setNormSet(norm_set_string);
-    tokenizer.setParagraphDetection(paragraphdetection);
-    tokenizer.setQuoteDetection(quotedetection);
-    tokenizer.setNormalization( normalization );
-    tokenizer.setInputEncoding( inputEncoding );
-    tokenizer.setFiltering(dofiltering);
-    tokenizer.setLangDetection(do_language_detect);
-    tokenizer.setPunctFilter(dopunctfilter);
-    tokenizer.setInputClass(inputclass);
-    tokenizer.setOutputClass(outputclass);
-    tokenizer.setXMLOutput(xmlout, docid);
-    tokenizer.setXMLInput(xmlin);
-    tokenizer.setTextRedundancy(redundancy);
 
     if (xmlin) {
-      folia::Document doc;
-      doc.readFromFile(ifile);
-      tokenizer.tokenize(doc);
-      *OUT << doc << endl;
-    } else {
+      folia::Document *doc = tokenizer.tokenize_folia( ifile );
+      if ( doc ){
+	*OUT << doc;
+	OUT->flush();
+	delete doc;
+      }
+    }
+    else {
       tokenizer.tokenize( *IN, *OUT );
       if ( OUT != &cout )
 	delete OUT;
